IF DB_ID('TestDB') IS NOT NULL
BEGIN
	ALTER DATABASE TestDB SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE TestDB;
END
GO

CREATE DATABASE [TestDB]
 
USE TestDB
GO

CREATE TABLE [dbo].[EmpSalary]
( [Department] [nchar](10) NULL,
  [EmpName] [nchar](10) NULL,
  [Salary] [int] NULL ) ON [PRIMARY]
GO

--Insert some test data to apply these functions
INSERT INTO [EmpSalary] VALUES ('IT','Emp1',87)
INSERT INTO [EmpSalary] VALUES ('IT','Emp2',45)
INSERT INTO [EmpSalary] VALUES ('IT','Emp3',89)
INSERT INTO [EmpSalary] VALUES ('IT','Emp4',87)
INSERT INTO [EmpSalary] VALUES ('OPS','Emp1',19)
INSERT INTO [EmpSalary] VALUES ('OPS','Emp2',33)
INSERT INTO [EmpSalary] VALUES ('OPS','Emp3',89)
INSERT INTO [EmpSalary] VALUES ('OPS','Emp4',89)


SELECT  
	Department,
	EmpName,
	Salary,
	PERCENT_RANK() OVER (ORDER BY [Salary]) as Percent_Rank,  
	CUME_DIST() OVER (ORDER BY [Salary]) as CUME_DIST   
FROM [EmpSalary]
ORDER BY [Salary]
GO 

/*  Cum_DIST Explained
We have a total of 8 records in the table and there are 5 distinct salary VALUES , so CUME_DIST VALUES  are determined as follows
For row 1 there is 1 row with that value or lower, so CUME_DIST = 1/8 or 0.125
For row 2 there are 2 rows with that value or lower, so CUME_DIST = 2/8 or 0.25
For row 3 there are 3 rows with that value or lower, so CUME_DIST = 3/8 or 0.375
For rows 4 and 5 there are 5 rows with that value or lower, so CUME_DIST = 5/8 or 0.625
For rows 6, 7 and 8 there are 8 rows with that value or lower, so CUME_DIST = 8/8 or 1
*/

/* PERCENT_RANK Explained
For row 1, based on the above, = 0
For row 2, the value will be (1/7) * 1 (row) = 0.1428571428571429
For row 3, the value will be (1/7) * 2 (rows) = 0.2857142857142857
For rows 4 and 5 the value will be (1/7) * 3 (rows) = 0.4285714285714287
For rows 6, 7 and 8 the value will be (1/7) * 5 (rows) = 0.7142857142857143
*/



-- With Partitioning

SELECT  
	Department,
	EmpName,
	Salary,
	PERCENT_RANK() OVER(PARTITION BY [Department] ORDER BY [Salary]) as Percent_Rank,  
	CUME_DIST() OVER(PARTITION BY [Department] ORDER BY [Salary])  as CUME_DIST  
FROM [EmpSalary]
ORDER BY[Department] , [Salary]
GO 