SELECT
	TransactionID,
	ProductID,
	TransactionDate,
	Quantity,
	ActualCost
FROM Production.TransactionHistory
ORDER BY TransactionDate DESC OFFSET 0 ROWS FETCH NEXT 20 ROWS ONLY;


-- CTE comparison

WITH Paging_CTE AS
(
SELECT
	TransactionID,
	ProductID,
	TransactionDate,
	Quantity,
	ActualCost,
	ROW_NUMBER() OVER (ORDER BY TransactionDate DESC) AS RowNumber
FROM
Production.TransactionHistory
)
SELECT
	TransactionID,
	ProductID,
	TransactionDate,
	Quantity,
	ActualCost
FROM Paging_CTE
WHERE RowNumber > 400 AND RowNumber <= 500


-- Specifying values programatticaly
DECLARE @OffsetRows tinyint = 0,
		@FetchRows tinyint = 20;
SELECT
	TransactionID,
	ProductID,
	TransactionDate,
	Quantity,
	ActualCost
FROM  Production.TransactionHistory
ORDER BY TransactionDate DESC OFFSET @OffsetRows ROWS FETCH NEXT @FetchRows ROWS ONLY;


DECLARE @OffsetRows tinyint = 0,
		@FetchRows tinyint = 20;


DECLARE @PageID INT = 1135;
DECLARE @RowsPerPage INT = 100;

DECLARE @MaxPageCount INT;

SELECT @MaxPageCount = COUNT(*)/@RowsPerPage
FROM  Production.TransactionHistory

SELECT
	TransactionID,
	ProductID,
	TransactionDate,
	Quantity,
	ActualCost
FROM  Production.TransactionHistory
ORDER BY TransactionDate DESC OFFSET IIF(@PageID > @MaxPageCount, (@MaxPageCount)*@RowsPerPage, (@PageID-1)*@RowsPerPage) ROWS FETCH NEXT @RowsPerPage ROWS ONLY;


SET STATISTICS IO ON
SET STATISTICS TIME ON

-- Total row count in 2012 with LAST_VALUE()
;WITH Paging_CTE AS
(
SELECT
	TransactionID,
	ProductID,
	TransactionDate,
	Quantity,
	ActualCost,
	ROW_NUMBER() OVER (ORDER BY TransactionDate DESC) AS RowNumber
FROM
Production.TransactionHistory
)
SELECT
	TransactionID,
	ProductID,
	TransactionDate,
	Quantity,
	ActualCost,
	LAST_VALUE(RowNumber) OVER(ORDER BY RowNumber ROWS BETWEEN CURRENT ROW AND UNBOUNDED FOLLOWING) AS [RowCount]
FROM Paging_CTE
WHERE RowNumber > 113400 AND RowNumber <= 113500;



-- Total Row Count in SQL Server 2008 and earlier with MAX()
;WITH Paging_CTE AS
(
SELECT
	TransactionID,
	ProductID,
	TransactionDate,
	Quantity,
	ActualCost,
	ROW_NUMBER() OVER (ORDER BY TransactionDate DESC) AS RowNumber
FROM
Production.TransactionHistory
)
SELECT
	TransactionID,
	ProductID,
	TransactionDate,
	Quantity,
	ActualCost,
	MAX(RowNumber) AS [RowCount]
FROM Paging_CTE
WHERE RowNumber > 113400 AND RowNumber <= 113500
GROUP BY TransactionID,
	ProductID,
	TransactionDate,
	Quantity,
	ActualCost